//
//  SudokuGrid.m
//  Sudoku
//
//  Created by Benjamin Ragheb on 12/23/06.
//  Copyright 2008 Benjamin Ragheb. All rights reserved.
//

#import "SudokuGrid.h"

/*
 This is a completed Sudoku grid, which we use as seed data to generate new
 grids.  The shuffle: method takes care of transforming the grid into a new
 game.
 */
const unsigned char kSeedData[] = { 
	1,2,3, 4,5,6, 7,8,9,
	4,5,6, 7,8,9, 1,2,3,
	7,8,9, 1,2,3, 4,5,6,
	2,3,4, 5,6,7, 8,9,1,
	5,6,7, 8,9,1, 2,3,4,
	8,9,1, 2,3,4, 5,6,7,
	3,4,5, 6,7,8, 9,1,2,
	6,7,8, 9,1,2, 3,4,5,
	9,1,2, 3,4,5, 6,7,8
};

/*
 Data is stored in a 9x9 array of bytes.  The lower four bits are used to 
 store the value at that cell (a digit 1-9 or clear).  The upper four fits
 are used to indicate if that value is in error (a duplicate value exists
 in the same row, column, or 3x3 square).
 */
enum {
	kDataValueMask = 0x0f,
	kDataAllErrorsMask = 0x70,
	kDataRowErrorMask = 0x10,
	kDataColumnErrorMask = 0x20,
	kDataSquareErrorMask = 0x40,
	kDataReadOnlyMask = 0x80, // not used
};

@implementation SudokuGrid

- (id)init {
	self = [super init];
	if (self) {
		[self resetWithDifficulty:50];
	}
	return self;
}

- (int)dataOffsetForRow:(int)row andColumn:(int)col {
	if (row < 0 || row > 8 || col < 0 || col > 8) {
		@throw NSInvalidArgumentException;
	}
	return row * 9 + col;
}

// Fills an array with the offsets of all the cells in the specified row.
- (void)getDataOffsets:(int *)offsets forRow:(int)row {
	int i;
	
	for (i = 0; i < 9; i++) {
		offsets[i] = [self dataOffsetForRow:row andColumn:i];
	}
}

// Fills an array with the offsets of all the cells in the specified column.
- (void)getDataOffsets:(int *)offsets forColumn:(int)col {
	int i;
	
	for (i = 0; i < 9; i++) {
		offsets[i] = [self dataOffsetForRow:i andColumn:col];
	}
}

// Fills an array with the offsets of all the cells in the specified 3x3 square.
- (void)getDataOffsets:(int *)offsets forSquareAtRow:(int)row andColumn:(int)col {
	int firstRow = row - (row % 3);
	int firstCol = col - (col % 3);
	
	int offset = [self dataOffsetForRow:firstRow andColumn:firstCol];
	int i = 0;
	while (i < 9) {
		offsets[i++] = offset + 0;
		offsets[i++] = offset + 1;
		offsets[i++] = offset + 2;
		offset += 9;
	}
}

- (void)verifyDataAtOffsets:(int *)offsets withMask:(unsigned char)mask {
	int counts[10];
	int i;
	int offset;
	unsigned char value;
	
	// pass 1: count the number of occurrences of each value
	bzero(counts, 10 * sizeof(int));
	for (i = 0; i < 9; i++) {
		offset = offsets[i];
		value = kDataValueMask & data[offset];
		counts[value]++;
	}
	// pass 2: update the error bits of each cell
	for (i = 0; i < 9; i++) {
		offset = offsets[i];
		value = kDataValueMask & data[offset];
		if (counts[value] > 1) {
			data[offset] |= mask;
		} else {
			data[offset] &= ~mask;
		}
	}
}

- (void)verifyRow:(int)row {
	int offsets[9];
	[self getDataOffsets:offsets forRow:row];
	[self verifyDataAtOffsets:offsets withMask:kDataRowErrorMask];
}

- (void)verifyColumn:(int)col {
	int offsets[9];
	[self getDataOffsets:offsets forColumn:col];
	[self verifyDataAtOffsets:offsets withMask:kDataColumnErrorMask];
}

- (void)verifySquareAtRow:(int)row andColumn:(int)col {
	int offsets[9];
	[self getDataOffsets:offsets forSquareAtRow:row andColumn:col];
	[self verifyDataAtOffsets:offsets withMask:kDataSquareErrorMask];
}

- (void)verifyGrid {
	int i, row, col;
	
	for (i = 0; i < 9; i++) {
		[self verifyRow:i];
		[self verifyColumn:i];
	}
	for (row = 0; row < 9; row += 3) {
		for (col = 0; col < 9; col += 3) {
			[self verifySquareAtRow:row andColumn:col];
		}
	}
}

/*
 This is a generic method to swap values in two different sets of cells.
 The sets of cells are specified by an array of array indexes. 
 */
- (void)exchangeDataAtOffsets:(int *)firstOffsets andOffsets:(int *)secondOffsets {
	unsigned char value;
	int i;
	for (i = 0; i < 9; i++) {
		value = data[firstOffsets[i]];
		data[firstOffsets[i]] = data[secondOffsets[i]];
		data[secondOffsets[i]] = value;
	}
}

/*
 This method shuffles a sudoku grid while preserving its validity.  The rows of
 a sudoku grid can be grouped into 3 sets.  The first set consists of rows 1-3,
 the second set of rows 4-6, and the third set of rows 7-9.  Any two rows 
 within the same set can be swapped without changing the validity of the grid.
 The same rule applies to columns.
 
 This method randomly picks two rows/columns within the same set and swaps them.
 It repeats this process 42 times to randomly shuffle the entire puzzle.
 
 Transformations not used:
 - swapping symbols (e.g., swapping 2 and 5 everywhere)
 - swapping row/column sets (e.g. swapping rows 1-3 with rows 7-9)
 - grid rotation
 */
- (void)shuffle {
	int loopCount;
	int offsetsA[9];
	int offsetsB[9];
	
	for (loopCount = 0; loopCount < 42; loopCount++) {
		int set = random() % 3;
		int rcA = (set * 3) + (random() % 3);
		int rcB = (set * 3) + (random() % 3);
		
		if (rcA == rcB) continue;
		if (random() % 2) {
			[self getDataOffsets:offsetsA forRow:rcA];
			[self getDataOffsets:offsetsB forRow:rcB];
		} else {
			[self getDataOffsets:offsetsA forColumn:rcA];
			[self getDataOffsets:offsetsB forColumn:rcB];
		}
		[self exchangeDataAtOffsets:offsetsA andOffsets:offsetsB];
	}
	[self verifyGrid];
}

- (void)resetWithDifficulty:(int)difficulty {
	// 1. copy completed puzzle into the grid
	bcopy(kSeedData, data, (9 * 9) * sizeof(unsigned char));
	
	// 2. randomly shuffle the grid
	[self shuffle];

	// 3. randomly clear squares
	int i;
	
	for (i = 0; i < (9 * 9); i++) {
		if (random() % 100 < difficulty) {
			data[i] = 0;
		} else {
			data[i] |= kDataReadOnlyMask;
		}
	}
	
	// 4. update error bits in the grid
	[self verifyGrid];
}

- (void)resetWithDifficulty:(int)difficulty serial:(unsigned long)serial {
	srandom(serial);
	[self resetWithDifficulty:difficulty];
}

- (unsigned char)valueAtRow:(int)row andColumn:(int)col {
	int i = [self dataOffsetForRow:row andColumn:col];
	return kDataValueMask & data[i];
}

- (void)setValue:(unsigned char)value atRow:(int)row andColumn:(int)col {
	int i = [self dataOffsetForRow:row andColumn:col];
	data[i] = (kDataValueMask & value);
	[self verifyRow:row];
	[self verifyColumn:col];
	[self verifySquareAtRow:row andColumn:col];
}

- (BOOL)isInvalidAtRow:(int)row andColumn:(int)col {
	int i = [self dataOffsetForRow:row andColumn:col];
	return (kDataAllErrorsMask & data[i]) ? YES : NO;
}

- (BOOL)isReadOnlyAtRow:(int)row andColumn:(int)col {
	int i = [self dataOffsetForRow:row andColumn:col];
	return (kDataReadOnlyMask & data[i]) ? YES : NO;
}

@end
