//
//  SudokuGridView.m
//  Sudoku
//
//  Created by Benjamin Ragheb on 12/22/06.
//  Copyright 2008 Benjamin Ragheb. All rights reserved.
//

#import "SudokuGridView.h"
#import "SudokuGrid.h"

@implementation SudokuGridView

- (id)initWithFrame:(NSRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
		sudokuGrid = [[SudokuGrid alloc] init];
		selectedRow = -1;
		selectedColumn = -1;
    }
    return self;
}

- (void)dealloc {
	[sudokuGrid release];
	[super dealloc];
}

- (IBAction)reset:(id)sender {
	[sudokuGrid resetWithDifficulty:60];
	[[self window] orderFront:nil];
	[self setNeedsDisplay:YES];
}

- (IBAction)shuffle:(id)sender {
	[sudokuGrid shuffle];
	[self setNeedsDisplay:YES];
}

// computes the largest square that will fit inside the view bounds
- (NSRect)computeInscribedSquare {
	NSRect viewBounds = [self bounds];
	if (NSWidth(viewBounds) > NSHeight(viewBounds)) {
		// rectangle is wide
		float x = (NSWidth(viewBounds) / 2.0) - (NSHeight(viewBounds) / 2.0);
		return NSMakeRect(x, 0, NSHeight(viewBounds), NSHeight(viewBounds));
	} else {
		// rectange is tall
		float y = (NSHeight(viewBounds) / 2.0) - (NSWidth(viewBounds) / 2.0);
		return NSMakeRect(0, y, NSWidth(viewBounds), NSWidth(viewBounds));
	}
}

- (BOOL)acceptsFirstResponder {
	return YES;
}

- (void)mouseDown:(NSEvent *)theEvent {
	NSPoint mouseLoc = [self convertPoint:[theEvent locationInWindow] fromView:nil];
	NSRect squareRect = [self computeInscribedSquare];
	float cellWidth = NSWidth(squareRect) / 9.0;
	selectedColumn = (int)((mouseLoc.x - NSMinX(squareRect)) / cellWidth);
	selectedRow = (int)((mouseLoc.y - NSMinY(squareRect)) / cellWidth);
	[self setNeedsDisplay:YES];
}

- (void)moveLeft:(id)sender {
	if (selectedColumn < 1 || selectedColumn > 8) return;
	selectedColumn--;
	[self setNeedsDisplay:YES];
}

- (void)moveRight:(id)sender {
	if (selectedColumn < 0 || selectedColumn > 7) return;
	selectedColumn++;
	[self setNeedsDisplay:YES];
}

- (void)moveUp:(id)sender {
	if (selectedRow < 0 || selectedRow > 7) return;
	selectedRow++;
	[self setNeedsDisplay:YES];
}

- (void)moveDown:(id)sender {
	if (selectedRow < 1 || selectedRow > 8) return;
	selectedRow--;
	[self setNeedsDisplay:YES];
}

- (void)keyDown:(NSEvent *)theEvent {
	[self interpretKeyEvents:[NSArray arrayWithObject:theEvent]];
}

- (void)insertText:(NSString *)keys {
	// if no active selection, ignore
	if (selectedRow < 0 || selectedRow > 8) return;
	if (selectedColumn < 0 || selectedColumn > 8) return;

	int i;
	for (i = 0; i < [keys length]; i++) {
		unichar keyChar = [keys characterAtIndex:i];
		if (keyChar >= '0' && keyChar <= '9') {
			if ([sudokuGrid isReadOnlyAtRow:selectedRow	andColumn:selectedColumn]) {
				NSBeep();
			} else {
				[sudokuGrid setValue:(keyChar - '0') atRow:selectedRow andColumn:selectedColumn];
				[self setNeedsDisplay:YES];
			}
		}
	}
}

- (void)clearSelection {
	// if no active selection, ignore
	if (selectedRow < 0 || selectedRow > 8) return;
	if (selectedColumn < 0 || selectedColumn > 8) return;
	if ([sudokuGrid isReadOnlyAtRow:selectedRow	andColumn:selectedColumn]) {
		NSBeep();
	} else {
		[sudokuGrid setValue:0 atRow:selectedRow andColumn:selectedColumn];
		[self setNeedsDisplay:YES];
	}
}

- (void)deleteBackward:(id)sender {
	[self clearSelection];
}

- (void)deleteForward:(id)sender {
	[self clearSelection];
}

- (void)drawGridCellAtRow:(int)row andColumn:(int)col inRect:(NSRect)cellRect {
	if ([sudokuGrid isReadOnlyAtRow:row andColumn:col]) {
		[[NSColor colorWithCalibratedWhite:0.9 alpha:1.0] set];
		[NSBezierPath fillRect:cellRect];
	}

	if (row == selectedRow && col == selectedColumn) {
		[[NSColor selectedMenuItemColor] set];
		[NSBezierPath strokeRect:NSInsetRect(cellRect, 1.0, 1.0)];
	}
	
	int value = [sudokuGrid valueAtRow:row andColumn:col];
	if (value == 0) return;
	
	NSString *digit = [NSString stringWithFormat:@"%d", value];
	NSFont *font = [NSFont fontWithName:@"Helvetica" size:(NSHeight(cellRect) * 0.8)];
	NSColor *color;

	if ([sudokuGrid isInvalidAtRow:row andColumn:col]) {
		color = [NSColor redColor];
	} else {
		color = [NSColor blackColor];
	}
	
	NSDictionary *attrs = [NSDictionary dictionaryWithObjectsAndKeys:
		font, NSFontAttributeName,
		color, NSForegroundColorAttributeName,
		nil];
	
	NSSize digitSize = [digit sizeWithAttributes:attrs];
	NSPoint p = NSMakePoint(NSMidX(cellRect) - (digitSize.width / 2.0),
							NSMidY(cellRect) - (digitSize.height / 2.0));
	[digit drawAtPoint:p withAttributes:attrs];
}

- (void)drawRect:(NSRect)rect {
	NSRect squareRect = [self computeInscribedSquare];
	float cellWidth = NSWidth(squareRect) / 9.0;
    // fill background white
	[[NSColor whiteColor] set];
	[NSBezierPath fillRect:squareRect];
	// draw numbers
	int row, col;
	float x, y;
	for (col = 0; col < 9; col++) {
		x = NSMinX(squareRect) + (cellWidth * col);
		for (row = 0; row < 9; row++) {
			y = NSMinY(squareRect) + (cellWidth * row);
			NSRect cellRect = NSMakeRect(x, y, cellWidth, cellWidth);
			[self drawGridCellAtRow:row andColumn:col inRect:cellRect];
		}
	}
	// draw grid
	[[NSColor blackColor] set];
	int i;
	for (i = 0; i < 10; i++) {
		[NSBezierPath setDefaultLineWidth:((i % 3 == 0) ? 2.0 : 1.0)];
		NSPoint p1, p2;
		// draw horizontal
		p1 = NSMakePoint(NSMinX(squareRect) + (cellWidth * i), NSMinY(squareRect));
		p2 = NSMakePoint(NSMinX(squareRect) + (cellWidth * i), NSMaxY(squareRect));
		[NSBezierPath strokeLineFromPoint:p1 toPoint:p2];
		// draw vertical
		p1 = NSMakePoint(NSMinX(squareRect), NSMinY(squareRect) + (cellWidth * i));
		p2 = NSMakePoint(NSMaxX(squareRect), NSMinY(squareRect) + (cellWidth * i));
		[NSBezierPath strokeLineFromPoint:p1 toPoint:p2];
	}
}

@end
